@@ -44,7 +44,7 @@ class CommandWindow:
 
 	TODO: gtk 2.10 has a special wizard-widget, consider using it...'''
 
-	def __init__(self, account, jid):
+	def __init__(self, account, jid, commandnode=None):
 		'''Create new window.'''
 
 		# an account object
@@ -55,7 +55,7 @@ class CommandWindow:
 		self.commandlist=None	# a list of (commandname, commanddescription)
 
 		# command's data
-		self.commandnode = None
+		self.commandnode = commandnode
 		self.sessionid = None
 		self.dataform = None
 		self.allow_stage3_close = False
@@ -78,8 +78,12 @@ class CommandWindow:
 		self.data_form_widget.show()
 		self.sending_form_stage_vbox.pack_start(self.data_form_widget)
 
-		# setting initial stage
-		self.stage1()
+		if self.commandnode:
+			# Execute command
+			self.stage3()
+		else:
+			# setting initial stage
+			self.stage1()
 
 		# displaying the window
 		self.xml.signal_autoconnect(self)
@@ -293,7 +297,9 @@ class CommandWindow:
 		self.send_command(action)
 
 	def stage3_next_form(self, command):
-		assert isinstance(command, xmpp.Node)
+		if not isinstance(command, xmpp.Node):
+			self.stage5(error=_('Service sent malformed data'), senderror=True)
+			return
 
 		self.remove_pulsing()
 		self.sending_form_progressbar.hide()
@@ -303,6 +309,7 @@ class CommandWindow:
 		elif self.sessionid != command.getAttr('sessionid'):
 			self.stage5(error=_('Service changed the session identifier.'),
 				senderror=True)
+			return
 
 		self.form_status = command.getAttr('status')
 
@@ -480,11 +487,8 @@ class CommandWindow:
 		assert action in ('execute', 'prev', 'next', 'complete')
 
 		stanza = xmpp.Iq(typ='set', to=self.jid)
-		cmdnode = stanza.addChild('command', attrs={
-				'xmlns':xmpp.NS_COMMANDS,
-				'node':self.commandnode,
-				'action':action
-			})
+		cmdnode = stanza.addChild('command', namespace=xmpp.NS_COMMANDS, attrs={
+			'node':self.commandnode, 'action':action})
 
 		if self.sessionid:
 			cmdnode.setAttr('sessionid', self.sessionid)
@@ -492,7 +496,7 @@ class CommandWindow:
 		if self.data_form_widget.data_form:
 #			cmdnode.addChild(node=dataforms.DataForm(tofill=self.data_form_widget.data_form))
 			# FIXME: simplified form to send
-			
+
 			cmdnode.addChild(node=self.data_form_widget.data_form)
 
 		def callback(response):
@@ -511,8 +515,7 @@ class CommandWindow:
 		if self.sessionid and self.account.connection:
 			# we already have sessionid, so the service sent at least one reply.
 			stanza = xmpp.Iq(typ='set', to=self.jid)
-			stanza.addChild('command', attrs={
-					'xmlns':xmpp.NS_COMMANDS,
+			stanza.addChild('command', namespace=xmpp.NS_COMMANDS, attrs={
 					'node':self.commandnode,
 					'sessionid':self.sessionid,
 					'action':'cancel'
