@@ -71,6 +71,17 @@ class MessageControl:
 		or inactive (state is False)'''
 		pass  # Derived classes MUST implement this method
 
+	def minimizable(self):
+		'''Called to check if control can be minimized'''
+		# NOTE: Derived classes MAY implement this
+		return False
+
+	def safe_shutdown(self):
+		'''Called to check if control can be closed without loosing data.
+		returns True if control can be closed safely else False'''
+		# NOTE: Derived classes MAY implement this
+		return True
+
 	def allow_shutdown(self, method, on_response_yes, on_response_no,
 	on_response_minimize):
 		'''Called to check is a control is allowed to shutdown.
@@ -109,7 +120,7 @@ class MessageControl:
 		#return (label_str, None)
 		pass
 
-	def get_tab_image(self):
+	def get_tab_image(self, count_unread=True):
 		# Return a suitable tab image for display.
 		# None clears any current label.
 		return None
@@ -142,10 +153,8 @@ class MessageControl:
 
 		self.session = session
 
-		new_key = None
 		if session:
 			session.control = self
-			new_key = session.thread_id
 
 		if oldsession:
 			oldsession.control = None
@@ -160,9 +169,9 @@ class MessageControl:
 		if crypto_changed:
 			self.print_esession_details()
 
-	def send_message(self, message, keyID = '', type_ = 'chat',
-	chatstate = None, msg_id = None, composing_xep = None, resource = None,
-	user_nick = None):
+	def send_message(self, message, keyID='', type_='chat', chatstate=None,
+	msg_id=None, composing_xep=None, resource=None, user_nick=None, xhtml=None,
+	callback=None, callback_args=[]):
 		# Send the given message to the active tab.
 		# Doesn't return None if error
 		jid = self.contact.jid
@@ -173,22 +182,29 @@ class MessageControl:
 		conn = gajim.connections[self.account]
 
 		if not self.session:
-			sess = conn.find_controlless_session(jid)
+			if not resource:
+				if self.resource:
+					resource = self.resource
+				else:
+					resource = self.contact.resource
+			sess = conn.find_controlless_session(jid, resource=resource)
 
 			if self.resource:
 				jid += '/' + self.resource
 
 			if not sess:
-				sess = conn.make_new_session(jid)
+				if self.type_id == TYPE_PM:
+					sess = conn.make_new_session(jid, type_='pm')
+				else:
+					sess = conn.make_new_session(jid)
 
 			self.set_session(sess)
 
 		# Send and update history
-		return conn.send_message(jid, message, keyID, type_ = type_,
-			chatstate = chatstate, msg_id = msg_id,
-			composing_xep = composing_xep,
-			resource = self.resource, user_nick = user_nick,
-			session = self.session,
-			original_message = original_message)
+		conn.send_message(jid, message, keyID, type_=type_, chatstate=chatstate,
+			msg_id=msg_id, composing_xep=composing_xep, resource=self.resource,
+			user_nick=user_nick, session=self.session,
+			original_message=original_message, xhtml=xhtml, callback=callback,
+			callback_args=callback_args)
 
 # vim: se ts=3:
