@@ -34,6 +34,7 @@ import time
 import common
 import gtkgui_helpers
 import message_control
+import dialogs
 from chat_control import ChatControlBase
 
 from common import gajim
@@ -85,21 +86,21 @@ class MessageWindow(object):
 			orig_window.destroy()
 			del orig_window
 
-		# NOTE: we use 'connect_after' here because in 
+		# NOTE: we use 'connect_after' here because in
 		# MessageWindowMgr._new_window we register handler that saves window
 		# state when closing it, and it should be called before
 		# MessageWindow._on_window_delete, which manually destroys window
 		# through win.destroy() - this means no additional handlers for
 		# 'delete-event' are called.
-		id = self.window.connect_after('delete-event', self._on_window_delete)
-		self.handlers[id] = self.window
-		id = self.window.connect('destroy', self._on_window_destroy)
-		self.handlers[id] = self.window
-		id = self.window.connect('focus-in-event', self._on_window_focus)
-		self.handlers[id] = self.window
+		id_ = self.window.connect_after('delete-event', self._on_window_delete)
+		self.handlers[id_] = self.window
+		id_ = self.window.connect('destroy', self._on_window_destroy)
+		self.handlers[id_] = self.window
+		id_ = self.window.connect('focus-in-event', self._on_window_focus)
+		self.handlers[id_] = self.window
 
 		keys=['<Control>f', '<Control>g', '<Control>h', '<Control>i',
-			'<Control>l', '<Control>L', '<Control>n', '<Control>u', '<Control>v',
+			'<Control>l', '<Control>L', '<Control>n', '<Control>u',
 			'<Control>b', '<Control><Shift>Tab', '<Control>Tab', '<Control>F4',
 			'<Control>w', '<Control>Page_Up', '<Control>Page_Down', '<Alt>Right',
 			'<Alt>Left', '<Alt>a', '<Alt>c', '<Alt>m', '<Alt>t', 'Escape'] + \
@@ -116,12 +117,12 @@ class MessageWindow(object):
 		self.window.add_events(gtk.gdk.POINTER_MOTION_MASK)
 		self.alignment = self.xml.get_widget('alignment')
 
-		id = self.notebook.connect('switch-page',
+		id_ = self.notebook.connect('switch-page',
 			self._on_notebook_switch_page)
-		self.handlers[id] = self.notebook
-		id = self.notebook.connect('key-press-event',
+		self.handlers[id_] = self.notebook
+		id_ = self.notebook.connect('key-press-event',
 			self._on_notebook_key_press)
-		self.handlers[id] = self.notebook
+		self.handlers[id_] = self.notebook
 
 		# Remove the glade pages
 		while self.notebook.get_n_pages():
@@ -145,14 +146,7 @@ class MessageWindow(object):
 		else:
 			self.notebook.set_show_tabs(False)
 		self.notebook.set_show_border(gajim.config.get('tabs_border'))
-
-		# if GTK+ version < 2.10, use OUR way to reorder tabs (set up DnD)
-		if gtk.pygtk_version < (2, 10, 0) or gtk.gtk_version < (2, 10, 0):
-			self.hid = self.notebook.connect('drag_data_received',
-				self.on_tab_label_drag_data_received_cb)
-			self.handlers[self.hid] = self.notebook
-			self.notebook.drag_dest_set(gtk.DEST_DEFAULT_ALL, self.DND_TARGETS,
-				gtk.gdk.ACTION_MOVE)
+		self.show_icon()
 
 	def change_account_name(self, old_name, new_name):
 		if old_name in self._controls:
@@ -165,11 +159,19 @@ class MessageWindow(object):
 		if self.account == old_name:
 			self.account = new_name
 
+	def change_jid(self, account, old_jid, new_jid):
+		''' call then when the full jid of a contral change'''
+		if account not in self._controls:
+			return
+		if old_jid not in self._controls[account]:
+			return
+		if old_jid == new_jid:
+			return
+		self._controls[account][new_jid] = self._controls[account][old_jid]
+		del self._controls[account][old_jid]
+
 	def get_num_controls(self):
-		n = 0
-		for dict in self._controls.values():
-			n += len(dict)
-		return n
+		return sum(len(d) for d in self._controls.values())
 
 	def resize(self, width, height):
 		gtkgui_helpers.resize_window(self.window, width, height)
@@ -196,6 +198,32 @@ class MessageWindow(object):
 			# Destroy the window
 			return False
 
+		# Number of controls that will be closed and for which we'll loose data:
+		# chat, pm, gc that won't go in roster
+		number_of_closed_control = 0
+		for ctrl in self.controls():
+			if not ctrl.safe_shutdown():
+				number_of_closed_control += 1
+
+		if number_of_closed_control > 1:
+			def on_yes1(checked):
+				if checked:
+					gajim.config.set('confirm_close_multiple_tabs', False)
+				self.dont_warn_on_delete = True
+				for ctrl in self.controls():
+					if ctrl.minimizable():
+						ctrl.minimize()
+				win.destroy()
+
+			if not gajim.config.get('confirm_close_multiple_tabs'):
+				# destroy window
+				return False
+			dialogs.YesNoDialog(
+				_('You are going to close several tabs'),
+            _('Do you really want to close them all?'),
+				checktext=_('Do _not ask me again'), on_response_yes=on_yes1)
+			return True
+
 		def on_yes(ctrl):
 			if self.on_delete_ok == 1:
 				self.dont_warn_on_delete = True
@@ -213,7 +241,6 @@ class MessageWindow(object):
 			self.on_delete_ok -= 1
 
 		# Make sure all controls are okay with being deleted
-		ctrl_to_minimize = []
 		self.on_delete_ok = self.get_nb_controls()
 		for ctrl in self.controls():
 			ctrl.allow_shutdown(self.CLOSE_CLOSE_BUTTON, on_yes, on_no,
@@ -257,19 +284,15 @@ class MessageWindow(object):
 		xml = gtkgui_helpers.get_glade('message_window.glade', 'chat_tab_ebox')
 		tab_label_box = xml.get_widget('chat_tab_ebox')
 		widget = xml.get_widget('tab_close_button')
-		id = widget.connect('clicked', self._on_close_button_clicked, control)
-		control.handlers[id] = widget
+		id_ = widget.connect('clicked', self._on_close_button_clicked, control)
+		control.handlers[id_] = widget
 
-		id = tab_label_box.connect('button-press-event', self.on_tab_eventbox_button_press_event,
+		id_ = tab_label_box.connect('button-press-event', self.on_tab_eventbox_button_press_event,
 					control.widget)
-		control.handlers[id] = tab_label_box
+		control.handlers[id_] = tab_label_box
 		self.notebook.append_page(control.widget, tab_label_box)
 
-		# If GTK+ version >= 2.10, use gtk native way to reorder tabs
-		if gtk.pygtk_version >= (2, 10, 0) and gtk.gtk_version >= (2, 10, 0):
-			self.notebook.set_tab_reorderable(control.widget, True)
-		else:
-			self.setup_tab_dnd(control.widget)
+		self.notebook.set_tab_reorderable(control.widget, True)
 
 		self.redraw_tab(control)
 		if self.parent_paned:
@@ -337,16 +360,6 @@ class MessageWindow(object):
 				control._on_change_nick_menuitem_activate(None)
 			elif keyval == gtk.keysyms.u: # CTRL + u: emacs style clear line
 				control.clear(control.msg_textview)
-			elif keyval == gtk.keysyms.v: # CTRL + v: Paste into msg_textview
-				if not control.msg_textview.is_focus():
-					control.msg_textview.grab_focus()
-				# Paste into the msg textview
-				event = gtk.gdk.Event(gtk.gdk.KEY_PRESS)
-				event.window = self.window.window
-				event.time = int(time.time())
-				event.state = gtk.gdk.CONTROL_MASK
-				event.keyval = gtk.keysyms.v
-				control.msg_textview.emit('key_press_event', event)
 			elif control.type_id == message_control.TYPE_GC and \
 			keyval == gtk.keysyms.b: # CTRL + b
 				control._on_bookmark_room_menuitem_activate(None)
@@ -401,11 +414,35 @@ class MessageWindow(object):
 		elif keyval == gtk.keysyms.Escape and \
 				gajim.config.get('escape_key_closes'): # Escape
 			self.remove_tab(control, self.CLOSE_ESC)
+		return True
 
 	def _on_close_button_clicked(self, button, control):
 		'''When close button is pressed: close a tab'''
 		self.remove_tab(control, self.CLOSE_CLOSE_BUTTON)
 
+	def show_icon(self):
+		window_mode = gajim.interface.msg_win_mgr.mode
+		icon = None
+		if window_mode == MessageWindowMgr.ONE_MSG_WINDOW_NEVER:
+			ctrl = self.get_active_control()
+			if not ctrl:
+				return
+			icon = ctrl.get_tab_image(count_unread=False)
+		elif window_mode == MessageWindowMgr.ONE_MSG_WINDOW_ALWAYS:
+			pass # keep default icon
+		elif window_mode == MessageWindowMgr.ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER:
+			pass # keep default icon
+		elif window_mode == MessageWindowMgr.ONE_MSG_WINDOW_PERACCT:
+			pass # keep default icon
+		elif window_mode == MessageWindowMgr.ONE_MSG_WINDOW_PERTYPE:
+			if self.type_ == 'gc':
+				icon = gtkgui_helpers.load_icon('muc_active')
+			else:
+				# chat, pm
+				icon = gtkgui_helpers.load_icon('online')
+		if icon:
+			self.window.set_icon(icon.get_pixbuf())
+
 	def show_title(self, urgent=True, control=None):
 		'''redraw the window's title'''
 		if not control:
@@ -493,13 +530,10 @@ class MessageWindow(object):
 			bctrl = self.get_control(jid, ctrl.account)
 			# keep last_message_time around unless this was our last control with
 			# that jid
-			if not fctrl and not bctrl:
+			if not fctrl and not bctrl and \
+			fjid in gajim.last_message_time[ctrl.account]:
 				del gajim.last_message_time[ctrl.account][fjid]
 
-			# Disconnect tab DnD only if GTK version < 2.10
-			if gtk.pygtk_version < (2, 10, 0) or gtk.gtk_version < (2, 10, 0):
-				self.disconnect_tab_dnd(ctrl.widget)
-
 			self.notebook.remove_page(self.notebook.page_num(ctrl.widget))
 
 			del self._controls[ctrl.account][fjid]
@@ -579,6 +613,8 @@ class MessageWindow(object):
 			else:
 				status_img.set_from_pixbuf(tab_img.get_pixbuf())
 
+		self.show_icon()
+
 	def repaint_themed_widgets(self):
 		'''Repaint controls in the window with theme color'''
 		# iterate through controls and repaint
@@ -671,11 +707,7 @@ class MessageWindow(object):
 				yield ctrl
 
 	def get_nb_controls(self):
-		nb_ctrl = 0
-		for jid_dict in self._controls.values():
-			for ctrl in jid_dict.values():
-				nb_ctrl += 1
-		return nb_ctrl
+		return sum(len(jid_dict) for jid_dict in self._controls.values())
 
 	def move_to_next_unread_tab(self, forward):
 		ind = self.notebook.get_current_page()
@@ -703,7 +735,7 @@ class MessageWindow(object):
 				# If no composing contact found yet, check if this one is composing
 					first_composing_ind = ind
 			if ind == current:
-				break # a complete cycle without finding an unread tab 
+				break # a complete cycle without finding an unread tab
 		if found:
 			self.notebook.set_current_page(ind)
 		elif first_composing_ind != -1:
@@ -742,45 +774,45 @@ class MessageWindow(object):
 			control.msg_textview.grab_focus()
 
 	def _on_notebook_key_press(self, widget, event):
-		# Ctrl+PageUP / DOWN has to be handled by notebook
-		if (event.state & gtk.gdk.CONTROL_MASK and
-				event.keyval in (gtk.keysyms.Page_Down, gtk.keysyms.Page_Up)):
-			return False
 		# when tab itself is selected, make sure <- and -> are allowed for navigating between tabs
 		if event.keyval in (gtk.keysyms.Left, gtk.keysyms.Right):
 			return False
 
 		control = self.get_active_control()
+
+		if event.state & gtk.gdk.SHIFT_MASK:
+			# CTRL + SHIFT + TAB
+			if event.state & gtk.gdk.CONTROL_MASK and \
+			event.keyval == gtk.keysyms.ISO_Left_Tab:
+				self.move_to_next_unread_tab(False)
+				return True
+			# SHIFT + PAGE_[UP|DOWN]: send to conv_textview
+			elif event.keyval in (gtk.keysyms.Page_Down, gtk.keysyms.Page_Up):
+				control.conv_textview.tv.emit('key_press_event', event)
+				return True
+		elif event.state & gtk.gdk.CONTROL_MASK:
+			if event.keyval == gtk.keysyms.Tab: # CTRL + TAB
+				self.move_to_next_unread_tab(True)
+				return True
+			# Ctrl+PageUP / DOWN has to be handled by notebook
+			elif event.keyval == gtk.keysyms.Page_Down:
+				self.move_to_next_unread_tab(True)
+				return True
+			elif event.keyval == gtk.keysyms.Page_Up:
+				self.move_to_next_unread_tab(False)
+				return True
+		if event.keyval in (gtk.keysyms.Shift_L, gtk.keysyms.Shift_R,
+		gtk.keysyms.Control_L, gtk.keysyms.Control_R, gtk.keysyms.Caps_Lock,
+		gtk.keysyms.Shift_Lock, gtk.keysyms.Meta_L, gtk.keysyms.Meta_R,
+		gtk.keysyms.Alt_L, gtk.keysyms.Alt_R, gtk.keysyms.Super_L,
+		gtk.keysyms.Super_R, gtk.keysyms.Hyper_L, gtk.keysyms.Hyper_R):
+			return True
+
 		if isinstance(control, ChatControlBase):
 			# we forwarded it to message textview
 			control.msg_textview.emit('key_press_event', event)
 			control.msg_textview.grab_focus()
 
-	def setup_tab_dnd(self, child):
-		'''Set tab label as drag source and connect the drag_data_get signal'''
-		tab_label = self.notebook.get_tab_label(child)
-		tab_label.dnd_handler = tab_label.connect('drag_data_get',
-			self.on_tab_label_drag_data_get_cb)
-		self.handlers[tab_label.dnd_handler] = tab_label
-		tab_label.drag_source_set(gtk.gdk.BUTTON1_MASK, self.DND_TARGETS,
-			gtk.gdk.ACTION_MOVE)
-		tab_label.page_num = self.notebook.page_num(child)
-
-	def on_tab_label_drag_data_get_cb(self, widget, drag_context, selection,
-		info, time):
-		source_page_num = self.find_page_num_according_to_tab_label(widget)
-		# 8 is the data size for the string
-		selection.set(selection.target, 8, str(source_page_num))
-
-	def on_tab_label_drag_data_received_cb(self, widget, drag_context, x, y,
-		selection, type_, time):
-		'''Reorder the tabs according to the drop position'''
-		source_page_num = int(selection.data)
-		dest_page_num, to_right = self.get_tab_at_xy(x, y)
-		source_child = self.notebook.get_nth_page(source_page_num)
-		if dest_page_num != source_page_num:
-			self.notebook.reorder_child(source_child, dest_page_num)
-
 	def get_tab_at_xy(self, x, y):
 		'''Thanks to Gaim
 		Return the tab under xy and
@@ -822,12 +854,6 @@ class MessageWindow(object):
 				break
 		return page_num
 
-	def disconnect_tab_dnd(self, child):
-		'''Clean up DnD signals, source and dest'''
-		tab_label = self.notebook.get_tab_label(child)
-		tab_label.drag_source_unset()
-		tab_label.disconnect(tab_label.dnd_handler)
-
 ################################################################################
 class MessageWindowMgr(gobject.GObject):
 	'''A manager and factory for MessageWindow objects'''
@@ -995,8 +1021,11 @@ class MessageWindowMgr(gobject.GObject):
 
 		# Position and size window based on saved state and window mode
 		if not self.one_window_opened(contact, acct, type_):
-			self._resize_window(win, acct, type_)
-			self._position_window(win, acct, type_)
+			if gajim.config.get('msgwin-max-state'):
+				win.window.maximize()
+			else:
+				self._resize_window(win, acct, type_)
+				self._position_window(win, acct, type_)
 
 		self._windows[win_key] = win
 		return win
@@ -1068,6 +1097,7 @@ May be useful some day in the future?'''
 
 	def save_state(self, msg_win, width_adjust=0):
 		# Save window size and position
+		max_win_key = 'msgwin-max-state'
 		pos_x_key = 'msgwin-x-position'
 		pos_y_key = 'msgwin-y-position'
 		size_width_key = 'msgwin-width'
@@ -1106,6 +1136,9 @@ May be useful some day in the future?'''
 				gajim.config.set_per('accounts', acct, pos_y_key, y)
 
 		else:
+			win_maximized = msg_win.window.window.get_state() == \
+				gtk.gdk.WINDOW_STATE_MAXIMIZED
+			gajim.config.set(max_win_key, win_maximized)
 			width += width_adjust
 			gajim.config.set(size_width_key, width)
 			gajim.config.set(size_height_key, height)
